"""
.. module:: teamcontrol.config
   :platform: Unix, Windows
   :synopsis: Configuration reader module for teamcontrol.

.. moduleauthor:: Caio Romao <caioromao@gmail.com>

"""

__all__ = ["TeamControlConfig", "RuleError"]

import re

try:
    from configparser import RawConfigParser, NoOptionError
except ImportError:
    from ConfigParser import RawConfigParser, NoOptionError


class RuleError(Exception):
    pass


class TeamControlConfig(RawConfigParser):
    """Custom ConfigParser class with specifics for :mod:`teamcontrol`
    like support for lists and aliases.
    """

    def get(self, section, option):
        """Gets an option from a section, converting to set for certain
        option fields.

        For an example file as such:
            [teamname]
            members = foo, bar, foobar
            rules =

        We'd get it as such:
        >>> config = TeamControlConfig()
        >>> config.read('myconfig.cfg')
        >>> config.get('teamname', 'members')
        ["foo", "bar", "foobar"]

        :param section: section name.
        :param option: option name.
        :rytpe: set or str.
        """
        value = RawConfigParser.get(self, section, option)
        if option in ('members', 'rules'):
            value = set([self.resolve(option,alias.strip())
                         for alias in value.split(',')])
        return value

    def resolve(self, option, alias):
        """Checks sections "members" and "rules" for an alias
        and returns it's value or the alias name in case it doesn't
        exist.

        Given an example file as such:
            [members]
            foo = Somebody <someuser@somedomain.com>

            [teamname]
            members = foo, bar
            rules =

        Getting the "members" from the "teamname" section would expand
        the alias.
        >>> config.get('teamname', 'members')
        ["Somebody <someuser@somedomain.com", "bar"]

        :param option: option name.
        :param alias: alias name to search for.
        :rtype: str.
        """
        resolved_alias = alias
        try:
            resolved_alias = RawConfigParser.get(self, option, alias)
        except NoOptionError:
            pass

        if option == "rules":
            # Try to parse rule as a regular expression
            try:
                resolved_alias = re.compile(resolved_alias)
            except re.error as err:
                msg = 'Invalid regex "{}": {}'.format(resolved_alias, err)
                raise RuleError(msg)

        return resolved_alias

    def get_team(self, teamname):
        """Gets a team configuration, which is simply a section with
        the "members" and "rules" options set.

        The result is a :type:`tuple` where the first item is the
        members list of the given team and the second one is the
        list of rules associated to it.

        In case a team configuration is incomplete or the section
        doesn't exist, the result is :type:`None`.

        :param teamname: name of the team.
        :rtype: tuple or None.
        """
        if (not self.has_section(teamname) or
            not self.has_option(teamname, 'members') or
            not self.has_option(teamname, 'rules')):
            return None
        return self.get(teamname, 'members'), self.get(teamname, 'rules')

    def get_team_names(self):
        """Gets every section of the configuration file which is not
        used internally for aliases or configuration.

        :rtype: set.
        """
        return set(self.sections()) - set(('members', 'rules'))

    def get_teams(self):
        """Gets every team and configuration set.

        The result is a 3-:type:`tuple` with the team name and it's
        settings. Check :method:`get_team` for the settings fields.

        :rtype: list of 3-tuples.
        """
        teams = []
        for team in self.get_team_names():
            settings = self.get_team(team)
            if settings is None:
                continue
            members, rules = settings
            teams.append((team, members, rules))
        return teams
