"""
.. module:: teamcontrol.control
   :platform: Unix, Windows
   :synopsis: Main application controller

.. moduleauthor:: Caio Romao <caioromao@gmail.com>

"""


class TeamController():
    """Main workflow controlller for TeamControl.
    """

    def __init__(self, scm, config, team_name=None):
        """Constructor.

        :param scm: source control manager instance.
        :type scm: :class:`~teamcontrol.scm.SCM`
        :param config: configuration manager instance.
        :type config: :class:`~teamcontrol.config.TeamControlConfig`
        :param team_name: name of the default team to list
        """
        self._team_name = team_name
        self._team_users_map = {}
        self._rule_team_relation = []
        self.__scm = scm

        # build internal data structures
        for name, users, rules in config.get_teams():
            self._team_users_map[name] = set(users)
            for rule in rules:
                self._rule_team_relation.append((rule, name))

    def _get_interested_users(self, filename):
        """Retrieves a list of team members that have rules matching
        the given filename.

        :param filename: filename to test for matching rules.
        :rtype: set.
        """
        users = set()
        for rule, team in self._rule_team_relation:
            if rule.match(filename):
                users |= self._team_users_map[team]
        return users

    def get_emails(self, input_data):
        """Parses input data and retrieves a list of users based on
        team rules that match each filename extracted from the input
        data.

        If :attribute:`_team_name` is set (Through the
        :method:`contructor __init__`), include every member from it
        regardless of the given data.

        :param input_data: data to be parsed. Command-line parameters,
                           for example.
        :type input_data: list
        :rtype: set.
        """
        emails = set()
        if self._team_name is not None:
            emails = self._team_users_map.get(self._team_name, set())

        changed_files = self.__scm.get_files(input_data)
        for filename in changed_files:
            emails |= self._get_interested_users(filename)

        return emails
