"""
.. module:: teamcontrol.scm
   :platform: Unix, Windows
   :synopsis: Source Code Manager related tools

.. moduleauthor:: Caio Romao <caioromao@gmail.com>

"""

__all__ = ["create_manager", "Git", "SCM"]

import re

from teamcontrol.util import run_command
from abc import ABCMeta, abstractmethod, abstractproperty


class SCMConfigurationError(Exception):
    pass


class SCM(object):
    """Source Control Manager abstract class.

    This is the base class for every SCM back-end supported.
    """

    __metaclass__ = ABCMeta

    @abstractproperty
    def parser(self):
        """Propery to retrieve the parser responsible for handling
        and identifying each patch. Check :module:`teamcontrol.parser`
        for implementation details.
        """
        pass

    @abstractmethod
    def list_changed_files(self, data):
        """Highlevel method to treat a parsed data and retrieve the
        list of files modified by it.
        """
        pass

    @abstractmethod
    def get_user_settings(self):
        """Retrieve user configured settings backed by the current
        SCM (the `git-config` interface, for example).

        :rtype: dict.
        """
        pass

    @abstractmethod
    def should_ignore(self, data):
        """Check's whether a data (patch file, for example) should
        be parsed for modified files or not.

        :param data: data to be tested.
        :rtype: bool.
        """
        return False

    def get_files(self, input_data):
        """Higher level workflow method to retrieve the list of
        modified files given a list of data to be parsed.

        :param input_data: data to be parsed.
        :type input_data: list.
        :rtype: set.
        """
        files = set()
        for data in input_data:
            if self.should_ignore(data):
                continue

            parsed_data = self.parser.parse(data)
            files |= self.list_changed_files(parsed_data)
        return files


class Git(SCM):

    LS_FILES_CMD = r'git show --pretty=format: --name-only {0}'
    GET_TEAM_CMD = r'git config teamcontrol.team'
    GET_CONFIG_CMD = r'git config teamcontrol.config'

    IGNORE_RE = re.compile(r'^0+-')

    NO_TEAM_ERR = ("No configuration found for teamcontrol.team.\n"
                   "Set your team name by issuing:\n"
                   "$ git config teamcontrol.team MY_TEAM_NAME")
    NO_CONFIG_ERR = ("No configuration found for teamcontrol.config.\n"
                     "Set your configuration path by issuing:\n"
                     "$ git config teamcontrol.config /path/to/config")

    def __init__(self):
        """Constructor.
        """
        from teamcontrol.parser import MboxParser
        self.__parser = MboxParser()

    @property
    def parser(self):
        """Gets a parser for Git's mailbox files generated by
        `git-format-patch`.

        :rtype: :class:`teamcontrol.parser.MboxParser` instance
        """
        return self.__parser

    def should_ignore(self, filename):
        """Test a given mailbox file against :attribute:`IGNORE_RE`.

        Currently, the only ignored mailbox is the one generated for
        a cover letter (0000-cover-letter.patch).

        :param filename: mailbox file to be tested.
        :rtype: bool.
        """
        if self.IGNORE_RE.match(filename):
            return True
        return False

    def list_changed_files(self, sha):
        """Gets list of files modified by a given commit, which is
        retrieved by the :attribute:`parser` when parsing a mailbox
        file.
        """
        cmd = self.LS_FILES_CMD.format(sha)
        stdout = run_command(cmd)

        return set(line.strip() for line in stdout.split())

    def get_user_settings(self):
        """Retrieves useful confiration from the current git
        repository (or the global git config file).

        Currently, the important data is `teamcontrol.team`, which
        defines the team the user executing TeamControl belongs and
        `teamcontrol.config`, which is the path to the confiration
        file parsed by :class:`teamcontrol.config.TeamControlConfig`.

        :rtype: dict.
        """
        teamname = run_command(self.GET_TEAM_CMD)
        configpath = run_command(self.GET_CONFIG_CMD)

        if not teamname:
            raise SCMConfigurationError(self.NO_TEAM_ERR)

        if not configpath:
            raise SCMConfigurationError(self.NO_CONFIG_ERR)

        return dict(team=teamname, config=configpath)


def create_manager():
    """Creates a SCM instance.

    Ideally this fuction will check the current path and define
    which SCM implementation to return.

    :rtype: `SCM` instance.
    """
    # TODO: Implement SCM discovery heuristics?
    return Git()
